#! /bin/sh

###########################################################
#
# @file: config/saga-uninstall.sh(.in)
#
# @description:
#   used to uninstall SAGA components from the target location.
#   The target location is either specified at configure time, 
#   via --prefix, or at runtime, via the SAGA_LOCATION 
#   environment variable, which overrides the configure prefix.
#   The script evaluates the install.log file created by 
#   'make install'
#
# Copyright (c) 2009 Andre Merzky (andre@merzky.net)
# This file is part of the SAGA Engine.
#
# Contributed by Andre Merzky   <andre@merzky.net>
#
###########################################################

### LICENSE ###

SAGA_ROOT="/home/condor/execute/dir_2628/userdir/SAGA.lsu-cpp.engine"

SAGA_LOC="/home/condor/execute/dir_2628/userdir/SAGA.lsu-cpp.engine/etics-tmp/usr"
if test $SAGA_LOCATION != ""; then
  SAGA_LOC=$SAGA_LOCATION
fi


usage()
{
  cat <<EOF

  Usage:    saga-uninstall.sh

  The program removes all SAGA components from \$SAGA_LOCATION,
  or, of that is now known, from the configure prefix specified 
  when building SAGA. 

EOF
  exit $1
}

# we need no stupid arguments 
if test $# -gt 0;
then
  usage 1 1>&2
fi

# check if install.log exists
log=$SAGA_LOC/share/saga/install.log

if ! test -f $log; then
  echo "Could not find install log at $log."
  echo "Make sure SAGA_LOCATION is set, and SAGA was "
  echo "installed via 'make install'."
  usage 1 1>&2
fi

echo 
echo "=================================================================="
echo "=== Do you really want to remove your SAGA installation? [y/N] ==="
echo "=================================================================="
echo 
read answer
if ! test "$answer" = "y"; then
  echo "Exiting..."
  exit 0
fi

# first, copy the install.log to a safe place
cp $log /tmp/saga.install.log.$$
log=/tmp/saga.install.log.$$

# remove installed files
for file in `tac $log | grep '^f ' | cut -f 2- -d ' '`; do
  echo "removing file $file"
  rm -f $file
done

# deep remove copied dirs
for Dir in `tac $log | grep '^D ' | cut -f 2- -d ' '`; do
  echo "removing dir  $Dir"
  rm -rf $Dir
done

# remove installed dirs
for dir in `tac $log | grep '^d ' | cut -f 2- -d ' '`; do
  echo "removing dir  $dir"
  rmdir -p $dir >& /dev/null
done

# # remove install root
# echo "removing root $SAGA_LOC"
# rmdir -p $SAGA_LOC

# finally, remore the copyt of the install.log
rm -f $log

