#ifndef SAGA_PACKAGES_ISN_ENTITY_DATA_HPP
#define SAGA_PACKAGES_ISN_ENTITY_DATA_HPP
#include <string>
#include <vector>
#include <iostream>

#include <saga/saga/util.hpp>
#include <saga/saga/base.hpp>
#include <saga/saga/types.hpp>
#include <saga/saga/session.hpp>
#include <saga/saga/call.hpp>
#include <saga/saga/detail/attribute.hpp>
#include <saga/saga/packages/isn/config.hpp>

namespace saga { namespace impl { class entity_data; } }

namespace saga
{

  namespace isn {

/**
 * Read access to the key/value pairs of a service_description instance.
 * This class implements the
 * saga::attributes interface and
 * offers getter methods for the user to read key/value pairs defined by the
 * service publisher. Service publishers are completely free to define their own
 * key names. Access to the keys and values is through the
 * <code>saga::attributes</code> interface. The class provides no other
 * methods.
 */
  class SAGA_ISN_PACKAGE_EXPORT entity_data
    : public saga::object,
      public saga::detail::attribute<entity_data>
  {
     protected:
       friend struct saga::detail::attribute<entity_data>;
       friend class saga::impl::entity_data;

       explicit entity_data(saga::impl::entity_data* impl);

       using saga::detail::attribute<entity_data>::remove_attribute;
    
     public:
       void dump() const
       {
          std::vector<std::string> attribNames = list_attributes();

          std::vector<std::string>::const_iterator attribNamesIter;
          std::vector<std::string>::const_iterator attribNamesEnd =
             attribNames.end();

          for ( attribNamesIter = attribNames.begin();
                attribNamesIter != attribNamesEnd;
                ++attribNamesIter )
          {
             if ( !attribute_is_vector(*attribNamesIter) )
             {
                std::string attribValue = get_attribute(*attribNamesIter);
                std::cout << *attribNamesIter
                          << " : "
                          << attribValue
                          << std::endl;
             }

             else
             {
                std::vector<std::string> attribValues =
                  get_vector_attribute(*attribNamesIter);

                std::vector<std::string>::const_iterator attribValuesIter;
                std::vector<std::string>::const_iterator attribValuesEnd =
                   attribValues.end();

                for ( attribValuesIter = attribValues.begin();
                      attribValuesIter != attribValuesEnd;
                      ++attribValuesIter )
                {
                   std::cout << *attribNamesIter
                             << " : "
                             << *attribValuesIter
                             << std::endl;
                }
             }
          }
       }

       entity_data(void);
       explicit entity_data(saga::object const& o);

       using saga::detail::attribute<entity_data>::get_attribute;
       using saga::detail::attribute<entity_data>::get_vector_attribute;
       using saga::detail::attribute<entity_data>::set_attribute;
       using saga::detail::attribute<entity_data>::set_vector_attribute;

      ~entity_data (void);

      // default operator=() and copy ctor are ok
      entity_data &operator= (saga::object const& o);

   }; // entity_data
  } // namespace isn
} // namespace saga
////////////////////////////////////////////////////////////////////////
#endif
