#  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
# 
#  Distributed under the Boost Software License, Version 1.0. (See accompanying
#  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

ifndef SAGA_MK_INC_CONF_C
       SAGA_MK_INC_CONF_C = yes

###########################################################
#
# paths
#
SAGA_LOCATION = /usr

SAGA_BIN_ROOT              = bin
SAGA_SHS_ROOT              = bin
SAGA_LIB_ROOT              = lib
SAGA_INI_ROOT              = share/saga
SAGA_HDR_ROOT              = include
SAGA_MAK_ROOT              = share/saga/make
SAGA_M4_ROOT               = share/saga/config

SAGA_EXT_ROOT              = $(SAGA_ROOT)/external

ifdef SAGA_ROOT
  SAGA_TOOL_ROOT           = $(SAGA_ROOT)/config
else
  SAGA_TOOL_ROOT           = $(SAGA_LOCATION)/bin
endif

#
###########################################################

###########################################################
#
# Tools
#
CP                         = /bin/cp -f
LN                         = /bin/ln -s
LS                         = /bin/ls
MV                         = /bin/mv -f
RM                         = /bin/rm -f
SH                         = /bin/sh
TR                         = /usr/bin/tr
AWK                        = /bin/awk
CAT                        = /bin/cat
CUT                        = /bin/cut
ENV                        = /bin/env
RPM                        = /usr/bin/rpmbuild
PWD                        = /bin/pwd
SED                        = /bin/sed
TAR                        = /bin/tar
DATE                       = /bin/date
GREP                       = /bin/grep
GZIP                       = /bin/gzip
ECHO                       = /bin/echo
FIND                       = /usr/bin/find
GZIP                       = /bin/gzip
SORT                       = /bin/sort
TEST                       = /usr/bin/test
TRUE                       = /bin/true
UNIQ                       = /usr/bin/uniq
EGREP                      = /bin/grep -E
MKDIR                      = /bin/mkdir -p
TOUCH                      = /bin/touch
FALSE                      = /bin/false
XARGS                      = /usr/bin/xargs
PDFLATEX                   = /usr/bin/pdflatex
DOXYGEN                    = /usr/bin/doxygen
DEVNULL                    = /dev/null

INSTALL                    = $(SAGA_TOOL_ROOT)/saga-install.sh
UNINSTALL                  = $(SAGA_TOOL_ROOT)/saga-uninstall.sh

MAKE                      += --no-print-directory
#
############################################################

###########################################################
#
# Compiler
#
SAGA_CPP                   = g++
SAGA_CPPFLAGS             += -E
SAGA_CPPINCS              += 
SAGA_CPPINCS_SYS          += 

SAGA_CC                    = gcc
ifneq "$(SAGA_DEBUG_MAKE)" "yes"
SAGA_CCFLAGS              += -g -O2
else
SAGA_CCFLAGS              += $(filter-out -O%,-g -O2)
endif

SAGA_CXX                   = g++
ifneq "$(SAGA_DEBUG_MAKE)" "yes"
SAGA_CXXFLAGS             += -g 
else
SAGA_CXXFLAGS             += $(filter-out -O%,-g )
endif

SAGA_GCC_VISIBILITY        = -fvisibility=default

SAGA_LD                    = g++
SAGA_LD_BIN                = g++
SAGA_LD_LIB                = g++
SAGA_LDFLAGS              += 
SAGA_LDFLAGS_SYS          += 

SAGA_LDFLAGS_READLINE      = 

SAGA_AR                    = /usr/bin/ar
SAGA_ARFLAGS              += rcs

SAGA_CCACHE                = 

SAGA_OSTYPE                = SAGA_LINUX

SAGA_BUILD_LIB_STATIC      = yes
SAGA_BUILD_LIB_DYNAMIC     = yes

SAGA_HAVE_LITE             = yes

# always add libdl
SAGA_LDFLAGS_SYS          += -ldl

# honor user provided flags
SAGA_CPPINCS              += $(SAGA_INC_FLAGS)
SAGA_CPPINCS_SYS          += $(SAGA_INC_FLAGS_SYS)
SAGA_ARFLAGS_SYS          += $(SAGA_AR_FLAGS)

#
###########################################################



###########################################################
#
# MacOS X specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "SAGA_APPLE"

  SAGA_LEXT                = dylib
  SAGA_AEXT                = a

ifneq "$(SAGA_DEBUG_MAKE)" "yes"
  SAGA_CXXFLAGS           += -Os
endif

  SAGA_CXXFLAGS           += -pipe -Wall -ansi -std=c++98
  SAGA_CXXFLAGS           += -Wno-trigraphs
  SAGA_CXXFLAGS_INT       += -fvisibility-inlines-hidden
  SAGA_LD_LIB             += -dynamiclib

  SAGA_CPPDEFS            += -D_FILE_OFFSET_BITS=64

endif
#
###########################################################

###########################################################
#
# IBM AIX 5 specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "SAGA_AIX"

  SAGA_LEXT                = so
  SAGA_AEXT                = a

  ifeq "$(SAGA_CC)" "gcc"  
    SAGA_LD_LIB           += -shared
    SAGA_LDFLAGS_SYS      += -pthread -Wl,-G -Wl,-brtl 
    SAGA_LDFLAGS_SYS      += -Wl,-binitfini:_GLOBAL__DI:_GLOBAL__DD
  endif
   
  ifeq "$(SAGA_CC)" "xlc"  
    SAGA_LDFLAGS_SYS      += -L/usr/lib/threads -G -brtl -bnoentry -lpthread -lc_r
    SAGA_CPPDEFS          += -D_THREAD_SAFE
    SAGA_CXXFLAGS         += -bnoentry
    SAGA_CCFLAGS          += -bnoentry
  endif
   
endif
#
###########################################################

###########################################################
#
# SunOS (Solaris)  specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "SAGA_SunOS"

  SAGA_LEXT                = so
  SAGA_AEXT                = a

  SAGA_LD_LIB           += -shared
  SAGA_LDFLAGS_SYS      += -pthread -lsocket

  ifeq "$(SAGA_CC)" "gcc"
    SAGA_CCFLAGS          += -pipe -fPIC
    SAGA_CCFLAGS          += -Wall -Wno-unknown-pragmas -ansi -std=c99
    SAGA_CXXFLAGS         += -fno-strict-aliasing -Wall -Wno-unknown-pragmas
  endif

  ifeq "$(SAGA_CXX)" "g++"
    SAGA_CXXFLAGS         += -pipe -fPIC
    SAGA_CXXFLAGS         += $(SAGA_GCC_VISIBILITY) -Wall -Wno-unknown-pragmas
    SAGA_CXXFLAGS         += -fno-strict-aliasing -ansi -std=c++98
  endif

endif
#
###########################################################


###########################################################
# 
# Linux specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "SAGA_LINUX"

  SAGA_LEXT                = so
  SAGA_AEXT                = a

  SAGA_LD_LIB             += -shared

  SAGA_CPPDEFS            += -D_FILE_OFFSET_BITS=64

  ifeq "$(SAGA_CC)" "gcc"
    SAGA_CCFLAGS          += -pipe -fPIC
    SAGA_CCFLAGS          += -Wall -Wno-unknown-pragmas -ansi -std=c99
    SAGA_CXXFLAGS         += -fno-strict-aliasing -Wall -Wno-unknown-pragmas
  endif

  ifeq "$(SAGA_CXX)" "g++"
    SAGA_CXXFLAGS         += -pipe -fPIC
    SAGA_CXXFLAGS         += $(SAGA_GCC_VISIBILITY) -Wall -Wno-unknown-pragmas
    SAGA_CXXFLAGS         += -fno-strict-aliasing -ansi -std=c++98
  endif

  ifeq "$(SAGA_CC)" "icc"
    SAGA_CCFLAGS          += -w1 -fpic -wd1125  
  endif

  ifeq "$(SAGA_CXX)" "icpc"
    SAGA_CXXFLAGS         += -w1 -fpic -wd1125
  endif

  ifeq "$(SAGA_LD)" "icpc"
    SAGA_LDFLAGS_SYS      += -static-libcxa -cxxlib-icc
  endif


endif
#
###########################################################

###########################################################
#
# well, we always need boost
#
ifeq "$(filter clean distclean confclean,$(MAKECMDGOALS))" ""
  ifdef SAGA_ROOT
    include $(SAGA_ROOT)/make/saga.config.boost.c.mk
  else
    include $(SAGA_LOCATION)/share/saga/make/saga.config.boost.c.mk
  endif
endif
#
###########################################################


###########################################################
#
# we also load any configuration checks we may know about
#
ifeq "$(filter clean distclean confclean,$(MAKECMDGOALS))" ""
  ifdef SAGA_ROOT
    include $(SAGA_ROOT)/make/saga.config.check.c.mk
  else
    include $(SAGA_LOCATION)/share/saga/make/saga.config.check.c.mk
  endif
endif
#
###########################################################

endif # guard

